﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using Microsoft.Kinect;

namespace KinectCam
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        KinectSensor myKinect;

        public MainWindow()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            if (KinectSensor.KinectSensors.Count == 0)
            {
                MessageBox.Show("No Kinects detected", "Camera Viewer");
                Application.Current.Shutdown();
            }

            try
            {
                myKinect = KinectSensor.KinectSensors[0];

                myKinect.ColorStream.Enable();

                myKinect.Start();
            }
            catch
            {
                MessageBox.Show("Kinect initialise failed", "Camera Viewer");
                Application.Current.Shutdown();
            }

            myKinect.ColorFrameReady += new EventHandler<ColorImageFrameReadyEventArgs>(myKinect_ColorFrameReady);
        }


        unsafe void updateImage(byte[] colorData)
        {
            int noOfPixelBytes = colorData.Length;

            fixed (byte* imageBase = colorData)
            {
                // For the duration of this block the pointer
                // imageBase points to the video image by
                byte* imagePosition = imageBase;
                byte* imageEnd = imageBase + noOfPixelBytes;

                int newValue;

                while (imagePosition != imageEnd)
                {
                    // Process Blue
                    newValue = *imagePosition + blueOffset;
                    if (newValue < 0) newValue = 0;
                    if (newValue > 255) newValue = 255;
                    *imagePosition = (byte)newValue;

                    // Move on to green
                    imagePosition++;
                    newValue = *imagePosition + greenOffset;
                    if (newValue < 0) newValue = 0;
                    if (newValue > 255) newValue = 255;
                    *imagePosition = (byte)newValue;

                    // Move on to red
                    imagePosition++;
                    newValue = *imagePosition + redOffset;
                    if (newValue < 0) newValue = 0;
                    if (newValue > 255) newValue = 255;
                    *imagePosition = (byte)newValue;

                    // skip past the alpha
                    imagePosition++;
                    imagePosition++;
                }
            }
        }


        byte[] colorData = null;
        WriteableBitmap colorImageBitmap = null;

        private int blueOffset = 0;
        private int redOffset = 0;
        private int greenOffset = 0;

        void myKinect_ColorFrameReady(object sender, ColorImageFrameReadyEventArgs e)
        {
            using (ColorImageFrame colorFrame = e.OpenColorImageFrame())
            {
                if (colorFrame == null) return;

                if (colorData == null)
                    colorData = new byte[colorFrame.PixelDataLength];

                colorFrame.CopyPixelDataTo(colorData);

                updateImage(colorData);

                if (colorImageBitmap == null)
                {
                    this.colorImageBitmap = new WriteableBitmap(
                        colorFrame.Width,
                        colorFrame.Height,
                        96,  // DpiX
                        96,  // DpiY
                        PixelFormats.Bgr32,
                        null);
                    kinectVideo.Source = colorImageBitmap;
                }

                this.colorImageBitmap.WritePixels(
                    new Int32Rect(0, 0, colorFrame.Width, colorFrame.Height),
                    colorData, // video data
                    colorFrame.Width * colorFrame.BytesPerPixel, // stride,
                    0   // offset into the array - start at 0
                    );

            }
        }

        private void redSlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            redOffset = (int)redSlider.Value;
        }

        private void greenSlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            greenOffset = (int)greenSlider.Value;
        }

        private void blueSlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            blueOffset = (int)blueSlider.Value;
        }

    }
}
