﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using Microsoft.Kinect;
using System.IO;
using System.Media;

namespace MotionDetector
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        KinectSensor myKinect;

        SoundPlayer alarmPlayer;

        public MainWindow()
        {
            InitializeComponent();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            if (KinectSensor.KinectSensors.Count == 0)
            {
                MessageBox.Show("No Kinects detected", "Camera Viewer");
                Application.Current.Shutdown();
            }

            try
            {
                myKinect = KinectSensor.KinectSensors[0];

                myKinect.DepthStream.Enable();
                myKinect.ColorStream.Enable();

                myKinect.Start();

                myKinect.DepthFrameReady += new EventHandler<DepthImageFrameReadyEventArgs>(myKinect_DepthFrameReady);
                myKinect.ColorFrameReady += new EventHandler<ColorImageFrameReadyEventArgs>(myKinect_ColorFrameReady);
            }
            catch
            {
                MessageBox.Show("Kinect initialise failed", "Depth Viewer");
                Application.Current.Shutdown();
            }

            alarmPlayer = new SoundPlayer("ding.wav");
        }

        #region Video Image processing

        byte[] colorData = null;
        WriteableBitmap colorImageBitmap = null;

        void myKinect_ColorFrameReady(object sender, ColorImageFrameReadyEventArgs e)
        {
            using (ColorImageFrame colorFrame = e.OpenColorImageFrame())
            {
                if (colorFrame == null) return;

                if (colorData == null)
                    colorData = new byte[colorFrame.PixelDataLength];

                colorFrame.CopyPixelDataTo(colorData);

                if (picturesToTake > 0)
                {
                    saveFrame(colorData, colorFrame.Width, colorFrame.Height);
                    picturesToTake--;
                }

                if (colorImageBitmap == null)
                {
                    this.colorImageBitmap = new WriteableBitmap(
                        colorFrame.Width,
                        colorFrame.Height,
                        96,  // DpiX
                        96,  // DpiY
                        PixelFormats.Bgr32,
                        null);
                }

                this.colorImageBitmap.WritePixels(
                    new Int32Rect(0, 0, colorFrame.Width, colorFrame.Height),
                    colorData, // video data
                    colorFrame.Width * colorFrame.BytesPerPixel, // stride,
                    0   // offset into the array - start at 0
                    );

                kinectVideo.Source = colorImageBitmap;
            }

            Dispatcher.Invoke(new Action(() => updateDisplay()));
        }

        #endregion



        #region Depth Image Processing

        short[] depthData = null;

        byte[] depthColorImage = null;

        WriteableBitmap depthImageBitmap = null;

        int minDistance;

        int distanceThreshold;

        void myKinect_DepthFrameReady(object sender, DepthImageFrameReadyEventArgs e)
        {
            using (DepthImageFrame depthFrame = e.OpenDepthImageFrame())
            {
                if (depthFrame == null) return;

                if (depthData == null)
                    depthData = new short[depthFrame.PixelDataLength];

                if (depthColorImage == null)
                    depthColorImage = new byte[depthFrame.PixelDataLength * 4];

                depthFrame.CopyPixelDataTo(depthData);

                int depthColorImagePos = 0;

                minDistance = 100000;

                for (int i = 0; i < depthFrame.PixelDataLength; i++)
                {
                    int depthValue = depthData[i] >> 3;
                    // Check for the invalid values

                    if (depthValue == myKinect.DepthStream.UnknownDepth)
                    {
                        depthColorImage[depthColorImagePos++] = 0; // Blue
                        depthColorImage[depthColorImagePos++] = 0; // Green
                        depthColorImage[depthColorImagePos++] = 255; // Red
                    }
                    else if (depthValue == myKinect.DepthStream.TooFarDepth)
                    {
                        depthColorImage[depthColorImagePos++] = 255; // Blue
                        depthColorImage[depthColorImagePos++] = 0; // Green
                        depthColorImage[depthColorImagePos++] = 0; // Red
                    }
                    else if (depthValue == myKinect.DepthStream.TooNearDepth)
                    {
                        depthColorImage[depthColorImagePos++] = 0; // Blue
                        depthColorImage[depthColorImagePos++] = 255; // Green
                        depthColorImage[depthColorImagePos++] = 0; // Red
                    }
                    else
                    {
                        if (depthValue < minDistance)
                        {
                            minDistance = depthValue;
                        }

                        byte depthByte = (byte)(255 - (depthValue >> 4));
                        depthColorImage[depthColorImagePos++] = depthByte; // Blue
                        depthColorImage[depthColorImagePos++] = depthByte; // Green
                        depthColorImage[depthColorImagePos++] = depthByte; // Red
                    }
                    // transparency
                    depthColorImagePos++;
                }

                // we now have a new array of color data

                if (depthImageBitmap == null)
                {
                    this.depthImageBitmap = new WriteableBitmap(
                        depthFrame.Width,
                        depthFrame.Height,
                        96,  // DpiX
                        96,  // DpiY
                        PixelFormats.Bgr32,
                        null);
                    kinectDepth.Source = depthImageBitmap;
                }

                this.depthImageBitmap.WritePixels(
                    new Int32Rect(0, 0, depthFrame.Width, depthFrame.Height),
                    depthColorImage, // video data
                    depthFrame.Width * 4, // stride,
                    0   // offset into the array - start at 0
                    );
            }
        }

        #endregion

        SolidColorBrush OKBrush = new SolidColorBrush(Colors.Green);
        SolidColorBrush AlarmBrush = new SolidColorBrush(Colors.Red);

        int picturesToTake = 0;

        void updateDisplay()
        {
            distanceValue.Content = minDistance.ToString();
            thresholdValue.Content = distanceThreshold.ToString();
            if (minDistance < distanceThreshold)
            {
                if (alarmLabel.Background != AlarmBrush)
                {
                    alarmPlayer.Play();
                    alarmLabel.Background = AlarmBrush;
                    picturesToTake = 5;
                }
            }
            else
            {
                alarmLabel.Background = OKBrush;
            }
        }

        int frameNo = 1;

        void saveFrame(byte[] videoImage, int width, int height)
        {
            string message = "";
            if (fileNameTextBox.Text == "")
            {
                message = "No filename set";
            }
            else
            {
                try
                {
                    BitmapSource pictureBitmap = null;

                    pictureBitmap = BitmapSource.Create(
                        width, height, 96, 96, PixelFormats.Bgr32, null,
                        videoImage, width * 4);

                    // Save image 
                    string destDir = System.IO.Path.GetDirectoryName(fileNameTextBox.Text);
                    string destFile = System.IO.Path.GetFileNameWithoutExtension(fileNameTextBox.Text);
                    string fileName = destDir + @"\" + destFile + frameNo.ToString() + ".jpg";

                    using (FileStream stream = new FileStream(fileName, FileMode.Create))
                    {
                        JpegBitmapEncoder encoder = new JpegBitmapEncoder();
                        encoder.Frames.Add(BitmapFrame.Create(pictureBitmap));
                        encoder.Save(stream);
                    }
                    frameNo++;
                    message = fileName + " saved OK";
                }
                catch
                {
                    message = "Save failed";
                }
            }

            Dispatcher.Invoke(new Action(() => { fileSaveStatusLabel.Content = message; }));
        }

        private void fileNameButton_Click(object sender, RoutedEventArgs e)
        {
            // Configure save file dialog box
            Microsoft.Win32.SaveFileDialog dlg = new Microsoft.Win32.SaveFileDialog();
            dlg.FileName = "Shot"; // Default file name
            dlg.DefaultExt = ".jpg"; // Default file extension
            dlg.Filter = "Pictures (.jpg)|*.jpg"; // Filter files by extension

            // Process save file dialog box results
            if (dlg.ShowDialog() == true)
            {
                // set the filename
                fileNameTextBox.Text = dlg.FileName;
            }
        }

        private void thresholdSlider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            distanceThreshold = (int)thresholdSlider.Value;
        }
    }
}
