using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

using Microsoft.Kinect;
using System.IO;
using System.Threading;

namespace KinectSoundProcessor
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            setupKinect();

            Thread audioThread = new Thread( new ThreadStart(pumpAudio));

            audioThread.IsBackground = true;

            audioThread.Priority = ThreadPriority.Highest;

            audioThread.Start();
        }

        string message;
        KinectSensor myKinect;

        protected bool setupKinect()
        {
            if (KinectSensor.KinectSensors.Count == 0)
            {
                message = "No Kinects detected";
                return false;
            }

            myKinect = KinectSensor.KinectSensors[0];

            try
            {
                myKinect.Start();
            }
            catch
            {
                message = "Kinect start failed";
                return false;
            }

            return true;
        }

        void pumpAudio()
        {
            // Create a buffer to hold the sound data
            byte[] soundSampleBuffer = new byte[1000];

            // Create a Kinect audio source and a sound input stream
            Stream kinectAudioStream;

            // Start the source and connect the stream to it
            kinectAudioStream = myKinect.AudioSource.Start();

            // Create an output soundeffect to play the audio
            DynamicSoundEffectInstance soundEffectOutput;
            soundEffectOutput = new DynamicSoundEffectInstance(16000, AudioChannels.Mono);

            // Start the soundeffect playing
            soundEffectOutput.Play();

            // Repeatedly read the sound in and then play it
            while (true)
            {
                int count = kinectAudioStream.Read(soundSampleBuffer, 0, soundSampleBuffer.Length);
                soundEffectOutput.SubmitBuffer(soundSampleBuffer, 0, count);
            }
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed ||
                Keyboard.GetState().IsKeyDown(Keys.Escape))
            {
                this.Exit();
            }

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);
            base.Draw(gameTime);
        }
    }
}
