﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;


using Microsoft.Kinect;
using System.IO;

namespace KinectSkeletonViewer
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {

        public MainWindow()
        {
            InitializeComponent();
        }

        KinectSensor myKinect;

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {

            // Check to see if a Kinect is available
            if (KinectSensor.KinectSensors.Count == 0)
            {
                MessageBox.Show("No Kinects detected", "Camera Viewer");
                Application.Current.Shutdown();
                return;
            }

            // Get the first Kinect on the computer
            myKinect = KinectSensor.KinectSensors[0];

            // Start the Kinect running and select the depth camera
            try
            {
                myKinect.SkeletonStream.Enable();
                myKinect.Start();
            }
            catch
            {
                MessageBox.Show("Kinect initialise failed", "Camera Viewer");
                Application.Current.Shutdown();
            }

            // connect a handler to the event that fires when new frames are available

            myKinect.SkeletonFrameReady += new EventHandler<SkeletonFrameReadyEventArgs>(myKinect_SkeletonFrameReady);

        }



        void myKinect_SkeletonFrameReady(object sender, SkeletonFrameReadyEventArgs e)
        {
            string message = "No Skeleton Data";
            string qualityMessage = "";

            // Remove the old skeleton
            skeletonCanvas.Children.Clear();

            Skeleton[] skeletons = null;

            using (SkeletonFrame frame = e.OpenSkeletonFrame())
            {
                if (frame != null)
                {
                    skeletons = new Skeleton[frame.SkeletonArrayLength];
                    frame.CopySkeletonDataTo(skeletons);
                }
            }

            if (skeletons == null) return;

            foreach (Skeleton skeleton in skeletons)
            {
                if (skeleton.TrackingState == SkeletonTrackingState.Tracked)
                {
                    Joint headJoint = skeleton.Joints[JointType.Head];
                    Joint hipCenter = skeleton.Joints[JointType.HipCenter];

                    if (skeleton.ClippedEdges == 0)
                    {
                        qualityMessage = "Good Quality";
                    }
                    else
                    {
                        if ((skeleton.ClippedEdges & FrameEdges.Bottom) != 0)
                            qualityMessage += "Move up ";
                        if ((skeleton.ClippedEdges & FrameEdges.Top) != 0)
                            qualityMessage += "Move down ";
                        if ((skeleton.ClippedEdges & FrameEdges.Right) != 0)
                            qualityMessage += "Move left ";
                        if ((skeleton.ClippedEdges & FrameEdges.Left) != 0)
                            qualityMessage += "Move right ";
                    }


                    SkeletonPoint headPosition = headJoint.Position;

                    Line backBone = new Line();
                    backBone.Stroke = new SolidColorBrush(Colors.Red);
                    backBone.StrokeThickness = 5;

                    ColorImagePoint headPoint = myKinect.MapSkeletonPointToColor(headJoint.Position, ColorImageFormat.RgbResolution640x480Fps30);
                    backBone.X1 = headPoint.X;
                    backBone.Y1 = headPoint.Y;

                    message = string.Format("Head: X:{0:000} Y:{1:000}",
                        headPoint.X,
                        headPoint.Y);

                    if (headJoint.TrackingState == JointTrackingState.Inferred)
                    {
                        message = message + " I";
                    }


                    ColorImagePoint hipPoint = myKinect.MapSkeletonPointToColor(hipCenter.Position, ColorImageFormat.RgbResolution640x480Fps30);
                    backBone.X2 = hipPoint.X;
                    backBone.Y2 = hipPoint.Y;

                    skeletonCanvas.Children.Add(backBone);

                }
            }
            StatusTextBlock.Text = message;
            QualityTextBlock.Text = qualityMessage;
        }
    }
}
