﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Runtime.InteropServices;

namespace MidiMan
{
class MIDIControl
{
    [DllImport("winmm.dll")]
    static extern uint midiOutOpen(ref IntPtr lphMidiOut, uint uDeviceID, IntPtr dwCallback, IntPtr dwInstance, uint dwFlags);

    [DllImport("winmm.dll")]
    static extern uint midiOutShortMsg(IntPtr hMidiOut, uint dwMsg);

    private IntPtr midiDevice = IntPtr.Zero;

    /// <summary>
    /// Create a new instance to manage the MIDI connection
    /// </summary>
    /// <param name="deviceNo">0 means local midi and 1 means extermal inteface</param>
    public MIDIControl(uint deviceNo)
    {
       uint result =  midiOutOpen(ref midiDevice, deviceNo, IntPtr.Zero, IntPtr.Zero, 0);
       if (result != 0) throw new Exception("MIDI error " + result.ToString());
    }


public void SendMIDIMessage(int command, int d1, int d2)
{
    byte[] data = new byte[4];

    // data [0] = command
    // data [1] = first byte of command data
    // data [2] = second byte of command data
    // data [3] = third byte of command data (not used)

    // assemble the message into the array
    data[0] = (byte)command;
    data[1] = (byte)d1;
    data[2] = (byte)d2;

    // send the message
    midiOutShortMsg(midiDevice, BitConverter.ToUInt32(data, 0));
}
}
}
